#!/bin/bash
set -x
# KVM users: please change this variable to point to the KVM virtual server
# that runs your CIT agent
kvm_guest_name=CLONEBIG

######## do not edit anything below here ########
# script to keep number of virtual CPUs of second level guest in sync with
# underlying LPAR. Can be executed in a z/VM guest, or on a KVM host

kvm_sig_name=kvm-ilmt-1
zvm_sig_name=zvm-ilmt-1
sig_path=/usr/local/bin
ch_cpu='/usr/sbin/chcpu'
ch_shut='/usr/sbin/chshut'
ls_reipl='/usr/sbin/lsreipl'
re_boot='/usr/sbin/reboot' 
script_name=`readlink -f $0`

function install {
# makes sure this script is called via crontab every 20 minutes
if [ "`whoami`" != "root" ]; then
	echo script must run as root. Copy into /usr/local/bin and run as root.
	exit
fi

crontab -l | grep $script_name > /dev/null
if [ "$?" -ne "0" ]; then
	echo adding $script_name to crontab
	(crontab -l ; echo "*/20 * * * * $script_name") | grep -v '^#' | sort - | uniq - | crontab -
fi

if [ "$1" == "zvm" ]; then
	if ! [ -x "$sig_path/$zvm_sig_name" ]; then
		mkdir -p $sig_path
		echo installing CIT signature
		ln -s /usr/bin/sleep $sig_path/$zvm_sig_name
	fi
	crontab -l | grep $zvm_sig_name > /dev/null
	if [ "$?" -ne "0" ]; then
	echo adding $zvm_sig_name to crontab
		(crontab -l ; echo "@reboot $sig_path/$zvm_sig_name 10000d") | grep -v '^#' | sort - | uniq - | crontab -
	fi
fi
}

function set_zvm_signature {
ps -ef | grep $zvm_sig_name | grep -v grep > /dev/null
if [ "$?" -ne "0" ]; then
	$sig_path_1/$zvm_sig_name 10000d &
fi
}

function zvm {
# installation requirements:
# * the guest must be able to define 64 virtual CPUs
$ch_cpu -r > /dev/null
$ch_cpu -d 1-63 > /dev/null 2>&1

lpar_cpus=$((`cat /proc/sysinfo | grep "LPAR CPUs Total" | cut -f2 -d':'`))
vm_cpus=$((`cat /proc/sysinfo |grep "VM00 CPUs Total"|cut -f2 -d':'`))

if [ "$lpar_cpus" -lt "$vm_cpus" ]; then
ipldev=`$ls_reipl | grep ^Device | cut -d':' -f2 | sed 's/0\.0\.//' | sed 's/[^A-Z^a-z^0-9]//g' | tr '[abcdef]' '[ABCDEF]'`
xlpar_cpus=$(printf "%X\n" $lpar_cpus)
last_cpu=$(($vm_cpus - 1))
xlast_cpu=$(printf "%X\n" $last_cpu)
$ch_shut reboot vmcmd "DETACH CPU $xlpar_cpus-$xlast_cpu" vmcmd "IPL $ipldev"
$re_boot
fi


if [ "$lpar_cpus" -gt "$vm_cpus" ]; then
vmcp def cpu $vm_cpus-$(($lpar_cpus - 1))
$ch_cpu -r > /dev/null
$ch_cpu -d 1-63 > /dev/null 2>&1
fi
}

function kvm {
# installation requirements:
# * 64 vcpus must be allowed (<vcpu> in domain)
# * please execute the following commands in the guest:
#      mkdir -p /usr/local/bin
#      ln -f /bin/sleep /usr/local/bin/kvm-ilmt-1
#      (crontab -l ; echo "@reboot /usr/local/bin/kvm-ilmt-1 10000d") | crontab -
# * make sure the guest name is listed in the $kvm_guest_name variable at
#   the top of the file.
if [ "$kvm_guest_name" == "your-kvm-ilmt-guest-name-here" ]; then
   echo
   echo ERROR:
   echo
   echo please edit $script_name and make sure the first variable
   echo at the beginnig of the file matches the name of your guest running
   echo the CIT agent.
   echo
   exit 2
fi

lpar_cpus=`cat /proc/sysinfo | grep "LPAR CPUs Total" | cut -f2 -d':'`
vm_cpus=`virsh dominfo $kvm_guest_name | grep "^CPU(s):" | cut -f2 -d':'`


if [ "$lpar_cpus" -ne "$vm_cpus" ]; then
   virsh setvcpus $kvm_guest_name $lpar_cpus --config
   virsh shutdown $kvm_guest_name
   sleep 60
   virsh destroy $kvm_guest_name
   virsh start $kvm_guest_name
fi
exit
}

testzvm=`cat /proc/sysinfo | grep 'VM00 Control Program' | cut -d':' -f2 | awk '{print $1;}'`
if [ "$testzvm" == "z/VM" ]; then
	install zvm
	set_zvm_signature
	zvm
	exit
elif grep --quiet kvmibm /etc/os-release; then
	install kvm
	kvm
	exit
fi
echo We are not running under z/VM. We also are not in a KVM hypervisor.
echo Exiting.
exit 1
